% notes page 9
% input: 
%	pi=(pi1 ... piK) the target multinomial
%	beta=(beta1 ... betaK) the Dirichlet prior parameters
%	The effort function E() (and its derivative) is hard-coded below
% output:
%	n=(n1 ... nK) the real-valued minimizer 

function optmultinomial()

% the teaching goal -------------------------------------------------
p=[0.1 0.3 0.6]'
%p=rand(19); p=p/sum(p)

% the learner's initial status (prior distribution) -----------------
b=[6 3 1]'
%b=rand(size(p))

% finding the optimal teaching sufficient statistics ----------------
% initial values and constraints for optimization
n0=rand(size(p))
f = @(n)theobjective(n,b,p);
options = optimset('GradObj', 'on');
% constraints: ni>=0, i=1..K
[n, fval]=fmincon(f,n0,[],[],[],[],zeros(size(n0)),Inf*ones(size(n0)),[],options);

% allocating sufficient statistics into teaching examples -----------
n = round(n)
sum(n)
[p n/sum(n)]

% compute the objective after teaching with these examples ----------
theobjective(n,b,p)

% compare the objective after teaching with iid N(target) examples --
for trial=1:100000,
  iid = mnrnd(sum(n),p)';
  iidobj(trial) = theobjective(iid,b,p);
end
iid
[min(iidobj) max(iidobj) mean(iidobj) std(iidobj)]

% compare the objective after teaching with target MLE examples -----
mle = round(sum(n)*p)
sum(mle)
[p mle/sum(mle)]
theobjective(mle, b,p)

% compute the objective after teaching with brute-force examples ----------
theobjective([317, 965, 1933]',b,p)

end

function [f, g] = theobjective(n, b, p)
K=length(b);
%c=0;
c=0.3;
Effort = c*sum(n);
dEffort = c;
f = -gammaln(sum(b)+sum(n)) + sum(gammaln(b+n)) - sum((b+n-1).*log(p)) + Effort;
g = -psi(sum(b)+sum(n)) + psi(b+n) - log(p) + dEffort;
end
